@@ -1,25 +1,26 @@
 #!/usr/bin/python3
 
 try:
-    import gettext
     from gi.repository import Gio, Gtk, GObject, Gdk, GdkPixbuf, GLib
     import tempfile
     import os
     import sys
     import zipfile
     import shutil
-    import cgi
+    import html
     import subprocess
     import threading
     import time
     import dbus
     from PIL import Image
     import datetime
+    import proxygsettings
 except Exception as detail:
     print(detail)
     sys.exit(1)
 
-from urllib.request import urlopen
+from http.client import HTTPSConnection
+from urllib.parse import urlparse
 
 try:
     import json
@@ -165,13 +166,10 @@ class Spice_Harvester(GObject.Object):
 
         if self.themes:
             self.install_folder = '%s/.themes/' % (home)
-            self.spices_directories = (self.install_folder, '%s/.themes/' % (home))
+            self.spices_directories = (self.install_folder, )
         else:
             self.install_folder = '%s/.local/share/cinnamon/%ss/' % (home, self.collection_type)
-            if self.collection_type == 'extension':
-                self.spices_directories = (self.install_folder, )
-            else:
-                self.spices_directories = ('/usr/share/cinnamon/%ss/' % self.collection_type, self.install_folder)
+            self.spices_directories = ('/usr/share/cinnamon/%ss/' % self.collection_type, self.install_folder)
 
         self._load_metadata()
 
@@ -381,21 +379,31 @@ class Spice_Harvester(GObject.Object):
         #interrupted.
         count = 0
         blockSize = 1024 * 8
+        parsed_url = urlparse(url)
+        host = parsed_url.netloc
         try:
-            with urlopen(url, timeout=15) as urlobj:
-                assert urlobj.getcode() == 200
+            proxy = proxygsettings.get_proxy_settings()
+            if proxy and proxy.get('https'):
+                connection = HTTPSConnection(proxy.get('https'), timeout=15)
+                connection.set_tunnel(host)
+            else:
+                connection = HTTPSConnection(host, timeout=15)
+            headers = { "Accept-Encoding": "identity", "Host": host, "User-Agent": "Python/3" }
+            connection.request("GET", parsed_url.path, headers=headers)
+            urlobj = connection.getresponse()
+            assert urlobj.getcode() == 200
 
-                totalSize = int(urlobj.info()['content-length'])
+            totalSize = int(urlobj.info()['content-length'])
 
-                while not self._is_aborted():
-                    data = urlobj.read(blockSize)
-                    count += 1
-                    if not data:
-                        break
-                    if not binary:
-                        data = data.decode("utf-8")
-                    outfd.write(data)
-                    ui_thread_do(reporthook, count, blockSize, totalSize)
+            while not self._is_aborted():
+                data = urlobj.read(blockSize)
+                count += 1
+                if not data:
+                    break
+                if not binary:
+                    data = data.decode("utf-8")
+                outfd.write(data)
+                ui_thread_do(reporthook, count, blockSize, totalSize)
         except Exception as e:
             raise e
 
@@ -440,7 +448,7 @@ class Spice_Harvester(GObject.Object):
             return False
 
         try:
-            return self.meta_map[uuid]["last-edited"] < self.index_cache[uuid]["last_edited"]
+            return int(self.meta_map[uuid]["last-edited"]) < self.index_cache[uuid]["last_edited"]
         except Exception as e:
             return False
 
@@ -450,7 +458,7 @@ class Spice_Harvester(GObject.Object):
         if not self.themes:
             enabled_list = self.settings.get_strv(self.enabled_key)
             for item in enabled_list:
-                if uuid in item:
+                if uuid in item.split(":"):
                     enabled_count += 1
         elif self.settings.get_string(self.enabled_key) == uuid:
             enabled_count = 1
@@ -628,11 +636,6 @@ class Spice_Harvester(GObject.Object):
         contents = os.listdir(folder)
 
         if not self.themes:
-            # ensure proper file permissions
-            for root, dirs, files in os.walk(folder):
-                for file in files:
-                    os.chmod(os.path.join(root, file), 0o755)
-
             # Install spice localization files, if any
             if 'po' in contents:
                 po_dir = os.path.join(folder, 'po')
@@ -648,6 +651,12 @@ class Spice_Harvester(GObject.Object):
             shutil.rmtree(dest)
         shutil.copytree(folder, dest)
 
+        if not self.themes:
+            # ensure proper file permissions
+            for root, dirs, files in os.walk(dest):
+                for file in files:
+                    os.chmod(os.path.join(root, file), 0o755)
+
         meta_path = os.path.join(dest, 'metadata.json')
         if self.themes and not os.path.exists(meta_path):
             md = {}
@@ -719,7 +728,7 @@ class Spice_Harvester(GObject.Object):
         markup = msg
         if detail is not None:
             markup += _("\n\nDetails:  %s") % (str(detail))
-        esc = cgi.escape(markup)
+        esc = html.escape(markup)
         dialog.set_markup(esc)
         dialog.show_all()
         response = dialog.run()
@@ -767,7 +776,14 @@ class Spice_Harvester(GObject.Object):
         enabled_extensions = self.settings.get_strv(self.enabled_key)
         new_list = []
         for enabled_extension in enabled_extensions:
-            if uuid not in enabled_extension:
+            if self.collection_type == 'applet':
+                enabled_uuid = enabled_extension.split(':')[3].strip('!')
+            elif self.collection_type == 'desklet':
+                enabled_uuid = enabled_extension.split(':')[0].strip('!')
+            else:
+                enabled_uuid = enabled_extension
+
+            if enabled_uuid != uuid:
                 new_list.append(enabled_extension)
         self.settings.set_strv(self.enabled_key, new_list)
 
